import { create } from 'zustand';
import { query } from '@/modules/query';
import { QueryLoginBrowser } from '@/query/query-login/query-login-browser';
import { QueryResources } from '@/query/query-resources/index';
import { toast } from 'react-toastify';
import { Chain } from '@kevisual/codemirror/utils';
export const chain = new Chain();

export const queryLogin = new QueryLoginBrowser({ query });
export const queryResources = new QueryResources({
  prefix: '/root/resources/',
});

export type MenuItem = {
  type: 'file' | 'directory';
  name?: string; // type === 'file'
  prefix?: string; // type === 'directory'
  etag?: string;
  lastModified?: string; // ISO date string
  size: number;
  url: string;
  path: string; // a/b/file.md
  pathname: string;
};

export type Menu = MenuItem[];

interface MenuState {
  menu: Menu;
  currentPath: string | null;
  isLoading: boolean;
  setMenu: (menu: Menu) => void;
  setCurrentPath: (path: string | null) => void;
  setLoading: (status: boolean) => void;
  onSaveText: () => Promise<any>;
}

export const useMenuStore = create<MenuState>((set, get) => ({
  menu: [],
  currentPath: null,
  isLoading: false,
  setMenu: (menu) => set({ menu }),
  setCurrentPath: (currentPath) => set({ currentPath }),
  setLoading: (isLoading) => set({ isLoading }),
  onSaveText: async () => {
    const { currentPath } = get();
    const content = chain.getContent();
    console.log('path', currentPath);
    console.log('Saving content:', content);
    const res = await postContent(currentPath!, content);
  },
}));

class Status {
  isInitialized = false;
  username = '';
}
const status = new Status();

export const init = async (resource: string = '', prefix: string = '') => {
  const { setMenu, setCurrentPath, setLoading } = useMenuStore.getState();
  let me = await queryLogin.checkLocalUser();
  const isInitialized = status.isInitialized;
  if (!isInitialized && me) {
    status.isInitialized = true;
    status.username = me.username!;
    queryResources.setUsername(status.username);
  }
  let recursive = true;
  const data = {};
  if (recursive) {
    data['recursive'] = recursive;
  }
  if (prefix) {
    queryResources.setPrefix(prefix);
  }
  const res = await queryResources.getList(resource, data);
  // @ts-ignore
  if (res?.status === 404) {
    toast.error('资源不存在，请检查路径是否正确');
    return;
  }
  if (res.code === 200) {
    const menu = res.data!.map((item: any) => {
      if (item.prefix) {
        item.type = 'directory';
      } else {
        item.type = 'file';
      }
      return item;
    });
    console.log('init menu', menu);
    if (recursive) {
      const obj: Record<string, any> = {};
      menu.forEach((item) => {
        const parts = item.path.split('/');
        const dirParts = parts.slice(0, -1);
        for (let i = 0; i < dirParts.length; i++) {
          const dir = dirParts.slice(0, i + 1).join('/');
          if (!dir) continue; // skip root
          obj[dir] = obj[dir] || { type: 'directory', path: dir };
        }
      });
      Object.keys(obj).forEach((key) => {
        const item = obj[key];
        menu.push(item);
      });
    }
    setMenu(menu);
    setCurrentPath('');
  }
};

export const getCurrentContent = async (currentPath: string): Promise<string | null> => {
  if (!currentPath) return null;
  const res = await queryResources.fetchFile(currentPath);
  console.log('getCurrentContent', res);
  if (res instanceof Response) {
    const status = res.status;
    if (status === 403) {
      toast.error('没有权限访问该资源。');
      return null;
    }
    if (status === 404) {
      toast.error('资源不存在，请检查路径是否正确');
      return null;
    }
    if (!res.ok) {
      console.error('Error fetching content:', res.statusText);
      toast.error(`获取内容失败: ${res.statusText}`);
      return null;
    }
    const content = await res.text();
    return content;
  }
  if (res.code === 200) {
    return res.data;
  } else {
    console.error('Error fetching content:', res.message);
    toast.error(`获取内容失败: ${res.message}`);
    return null;
  }
};

export const postContent = async (currentPath: string, content: string): Promise<boolean> => {
  if (!currentPath) return false;
  const res = await queryResources.uploadFile(currentPath, content);
  console.log('postContent', res);
  if (res.code === 200) {
    toast.success('内容保存成功');
    return true;
  } else {
    console.error('Error saving content:', res.message);
    toast.error(`保存内容失败: ${res.message}`);
    return false;
  }
};
